////////////////////////////////////////////////////////////////////
// 物理系パラメータ
////////////////////////////////////////////////////////////////////
// r6 への追加
// ・軌跡描画用データの準備
// ・コリオリ力と遠心力の追加（エネルギー保存則も拡張）
//


var PHYSLAB = {};
//実験室内の時間間隔
PHYSLAB.dt = 0.001; //1E-15*0.01;
//描画スキップ回数
PHYSLAB.skip = 50;
//実験室内の重力加速度
PHYSLAB.g = new THREE.Vector3();
//ばね定数
PHYSLAB.k = 0;
//静止摩擦係数
PHYSLAB.mu = 1.0;
//動摩擦係数
PHYSLAB.mu_star = 1.0;
//判定
PHYSLAB.delta = 0.01;
//空気抵抗係数（粘性抵抗）
PHYSLAB.gamma = 0.00;
//転がり抵抗係数
PHYSLAB.Crr = 0.00;
//反発係数
PHYSLAB.Er = 1.0;
//角運動量-並進運動量交換率
PHYSLAB.Et = 1.0;
//真空の誘電率
PHYSLAB.epsilon0 = 8.85418782E-12;
//電気素量
PHYSLAB.e = 1.60217657E-19;
//陽子質量
PHYSLAB.protonMass = 1.67262158E-27;
//電子質量
PHYSLAB.electronMass = 9.1093897E-31;
//電子ボルト
PHYSLAB.eV = 0.1602E-18; //[J]
//重力定数
PHYSLAB.G = 6.67408E-11; //[Nm^2/kg^2]
//レナード・ジョーンズポテンシャルで発生する力の強さ
PHYSLAB.RJ_epsilon = 1.0;
//レナード・ジョーンズポテンシャルの最下点距離
PHYSLAB.RJ_sigma = 1.0;

//結合の種類
PHYSLAB.SphereCollision = 1001;        //剛体球同士の衝突
PHYSLAB.SolidConnection = 1002;        //剛体棒による結合
PHYSLAB.LinearSpringConnection = 1003; //線形ばねによる結合
PHYSLAB.CoulombInteraction = 1004;     //クーロン相互作用
PHYSLAB.UniversalGravitation = 1005;   //万有引力相互作用
PHYSLAB.LennardJonesPotential = 1006;  //レナード・ジョーンズポテンシャル

//実験室内のスケール
PHYSLAB.DistanceScale = 1.0; //1E-9; //ナノメートル
PHYSLAB.TimeScale =     1.0; //1E-15; //アト秒
PHYSLAB.EnergyScale =   1.0; //PHYSLAB.eV; //電子ボルト

///////////////////////////////////
// 無極性原子クラスの定義
///////////////////////////////////
var Molecule = function ( parameter ) {
	parameter = parameter || {}

	//記録関連
	var record = parameter.record || {};
	this.record = {};
	this.record.enable = ( record.enable != undefined )? record.enable : false;
	this.record.interval = ( record.interval != undefined )? record.interval : PHYSLAB.skip;
	this.record.max = ( record.max != undefined )? record.max : 10000;

	//回転座標系（z軸周りのみに対応）
	var rotationalCoordinateSystem = parameter.rotationalCoordinateSystem || {};
	this.rotationalCoordinateSystem = {};
	this.rotationalCoordinateSystem.enable = ( rotationalCoordinateSystem.enable != undefined )? rotationalCoordinateSystem.enable : false;
	this.rotationalCoordinateSystem.alpha = ( rotationalCoordinateSystem.alpha != undefined )? rotationalCoordinateSystem.alpha : 0;
	this.rotationalCoordinateSystem.omega = ( rotationalCoordinateSystem.omega != undefined )? rotationalCoordinateSystem.omega : 2 * Math.PI / 100;

	//原子
	this.atoms = [];
	var atoms = parameter.atoms;
	for( var i = 0; i < atoms.length; i++ ){

		this.atoms[ i ] = {
			mass : atoms[ i ].mass || 1.0,
			radius : atoms[ i ].radius || 1.0,
			color : atoms[ i ].color || 0x000000,
			position : atoms[ i ].position || new THREE.Vector3(0, 0, i), //位置ベクトル
			velocity : atoms[ i ].velocity || new THREE.Vector3(0, 0, 0), //速度ベクトル
			theta : atoms[ i ].theta || new THREE.Vector3(0, 0, 0), //角度ベクトル
			omega : atoms[ i ].omega || new THREE.Vector3(0, 0, 0), //角速度ベクトル
			dynamic : ( atoms[ i ].dynamic !== undefined )? atoms[ i ].dynamic : true //運動の有無
		}

		//剛体球の慣性モーメント
		this.atoms[i].moment = 2.0/5.0 * this.atoms[i].mass * this.atoms[i].radius * this.atoms[i].radius;

		//固定されている場合
		if( !this.atoms[ i ].dynamic ){
			//一時保持
			this.atoms[ i ]._mass = this.atoms[ i ].mass;
			this.atoms[ i ].mass = Infinity;
			this.atoms[ i ]._moment = this.atoms[ i ].moment;
			this.atoms[ i ].moment = Infinity;

		}

		atoms[ i ].arrows = atoms[ i ].arrows || {}
		atoms[ i ].arrows.force = atoms[ i ].arrows.force || {};
		//物理量可視化用矢印オブジェクトのパラメータ
		this.atoms[i].arrows = {
			force : {
				available :( atoms[ i ].arrows.force.available !== undefined )? atoms[ i ].arrows.force.available : false,
				visible : ( atoms[ i ].arrows.force.visible !== undefined )? atoms[ i ].arrows.force.visible : true,
				color : ( atoms[ i ].arrows.force.color !== undefined )? atoms[ i ].arrows.force.color : this.atoms[i].color,
				maxLength : ( atoms[ i ].arrows.force.maxLength !== undefined )? atoms[ i ].arrows.force.maxLength : 10,
				minLength : ( atoms[ i ].arrows.force.minLength !== undefined )? atoms[ i ].arrows.force.minLength : 0.001,
				headLength : ( atoms[ i ].arrows.force.headLength !== undefined )? atoms[ i ].arrows.force.headLength : 1.0,
				headSize : ( atoms[ i ].arrows.force.headSize !== undefined )? atoms[ i ].arrows.force.headSize : 0.5,
				scale : ( atoms[ i ].arrows.force.scale !== undefined )? atoms[ i ].arrows.force.scale : 1 //1[N]当たりの長さ
			},
		};

		this.atoms[i].position_1 = new THREE.Vector3();     //位置ベクトル
		this.atoms[i].velocity_1 = new THREE.Vector3();     //速度ベクトル
		this.atoms[i].omega_1 = new THREE.Vector3();        //角速度ベクトル

		this.atoms[i].force = new THREE.Vector3();
		this.atoms[i].torque = new THREE.Vector3();

		this.atoms[i].collisionSphereForces = [];
		this.atoms[i].collisionSphereTorques = [];
		this.atoms[i].collisionSpheres = [];
		this.atoms[i].energy = {};


		this.atoms[i].records = {};
		this.atoms[i].records.position = [];
		this.atoms[i].records.velocity = [];
		this.atoms[i].records.omega = [];

	}



	//結合
	this.connects = [];
	var connects = parameter.connects;
	for( var i = 0; i < connects.length; i++ ){

		this.connects[ i ] = {
			pair1 : connects[ i ].pair1,
			pair2 : connects[ i ].pair2,
			type : connects[ i ].type,
			color : connects[ i ].color || 0xFFFFFF,
		}

		var n1 = this.connects[ i ].pair1;
		var n2 = this.connects[ i ].pair2;

		//剛体球同士の衝突
		if( this.connects[ i ].type == PHYSLAB.SphereCollision ){
			this.connects[ i ].Er = ( connects[ i ].Er != undefined )? connects[ i ].Er : PHYSLAB.Er; //並進反発係数
			this.connects[ i ].Et = ( connects[ i ].Et != undefined )? connects[ i ].Et : PHYSLAB.Et; //回転反発係数
		}
		//ばね弾性力
		if( this.connects[ i ].type == PHYSLAB.LinearSpringConnection ){
			this.connects[ i ].k  = ( connects[ i ].k  != undefined )? connects[ i ].k  : PHYSLAB.k; //ばね弾性力
			this.connects[ i ].L0 = ( connects[ i ].L0 != undefined )? connects[ i ].L0 : 0.0; //回転反発係数
			this.connects[ i ].radius = ( connects[ i ].radius  != undefined )? connects[ i ].radius : 0.2;  //ばねの半径
			this.connects[ i ].tube = ( connects[ i ].tube  != undefined )? connects[ i ].tube : 0.1; //管の半径
			this.connects[ i ].windingNumber = ( connects[ i ].windingNumber  != undefined )? connects[ i ].windingNumber : 10; //ばねの巻数
		}
		//クーロン相互作用
		if( this.connects[ i ].type == PHYSLAB.CoulombInteraction ){

 			//誘電率
 			this.connects[ i ].epsilon = ( connects[ i ].epsilon != undefined )? connects[ i ].epsilon : PHYSLAB.epsilon0;

			this.atoms[ n1 ].e = ( atoms[ n1 ].e != undefined )? atoms[ n1 ].e : PHYSLAB.e; //電荷
			this.atoms[ n2 ].e = ( atoms[ n2 ].e != undefined )? atoms[ n2 ].e : PHYSLAB.e; //電荷

		}
		//万有引力相互作用
		if( this.connects[ i ].type == PHYSLAB.UniversalGravitation ){

			//重力定数
			this.connects[ i ].G = ( connects[ i ].G != undefined )? connects[ i ].G : PHYSLAB.G;

		}

		//レナード・ジョーンズポテンシャル
		if( this.connects[ i ].type == PHYSLAB.LennardJonesPotential ){

			//力の強さ
			this.connects[ i ].epsilon = ( connects[ i ].epsilon != undefined )? connects[ i ].epsilon : PHYSLAB.LJ_epsilon;
			//ポテンシャルの最下点の距離
			this.connects[ i ].sigma = ( connects[ i ].sigma != undefined )? connects[ i ].sigma : PHYSLAB.LJ_sigma;

		}
		if( this.connects[ i ].type == PHYSLAB.SolidConnection ){

			var ni = this.connects[ i ].pair1;
			var nj = this.connects[ i ].pair2;

			var rij = new THREE.Vector3().subVectors(
				this.atoms[ ni ].position,
				this.atoms[ nj ].position
			);

			this.connects[ i ].L = rij.length();
/*
			this.atoms[ ni ]._moment = this.atoms[ ni ].moment;
			this.atoms[ ni ].moment = Infinity;
			this.atoms[ nj ]._moment = this.atoms[ nj ].moment;
			this.atoms[ nj ].moment = Infinity;
*/
		}

	}

	//剛体棒ペアの準備
	this.solidConnectionPairs = [];
	this.solidConnectionPairs[0] = [];
	var pairs = this.solidConnectionPairs;
	for( var i = 0; i < this.connects.length; i++ ){
		//結合タイプが剛体棒結合ではない場合はスキップ
		if( this.connects[ i ].type != PHYSLAB.SolidConnection ) continue;

		var ni = this.connects[ i ].pair1;
		var nj = this.connects[ i ].pair2;



		for( var k = 0; k < pairs.length; k++ ){

			if( pairs[ k ].indexOf( ni ) >= 0 ){

				if( pairs[ k ].indexOf( nj ) == -1 ) pairs[ k ].push( nj );
				else break;

			}else if( pairs[ k ].indexOf( nj ) >= 0 ){

				if( pairs[ k ].indexOf( ni ) == -1 ) pairs[ k ].push( ni );
				else break;

			}else if( k == pairs.length - 1 ){

				pairs.push( [ ni, nj ] );

			}

		}

	}

	this.collisionMode = false;

	this.step = 0;

//	this.checkCollision( PHYSLAB.dt );
//	this.calculateForce( this.atoms );

	for( var i = 0; i < this.atoms.length; i++ ) {
		for( var j = 0; j < this.atoms.length; j++ ) {
			this.atoms[ i ].collisionSphereForces[ j ] = new THREE.Vector3();
			this.atoms[ i ].collisionSphereTorques[ j ] = new THREE.Vector3();
		}
	}

	//
	this.recordResults();

};

//運動の記録
Molecule.prototype.recordResults = function( index ){

	if( !this.record.enable ) return;
	if( this.step % this.record.interval != 0 ) return;

	if( index != undefined ){

		for( var i = 0; i < this.atoms.length; i++ ){

			this.atoms[ i ].records.position[ index ] = this.atoms[ i ].position.clone();
			this.atoms[ i ].records.velocity[ index ] = this.atoms[ i ].velocity.clone();
			this.atoms[ i ].records.omega[ index ] = this.atoms[ i ].omega.clone();

		}


	} else {

		for( var i = 0; i < this.atoms.length; i++ ){

			this.atoms[ i ].records.position.push( this.atoms[ i ].position.clone() );
			this.atoms[ i ].records.velocity.push( this.atoms[ i ].velocity.clone() );
			this.atoms[ i ].records.omega.push( this.atoms[ i ].omega.clone() );

		}

	}

}

//RK4法アルゴリズムによる時間発展
Molecule.prototype.timeEvolution =  function ( dt ) {

	//剛体球同士の衝突判定
	this.checkCollision( dt );


	this.rotationalCoordinateSystem.omega_1 = this.rotationalCoordinateSystem.omega;
	//
	for( var i = 0; i < this.atoms.length; i++ ){
		this.atoms[ i ].position_1.copy( this.atoms[ i ].position );
		this.atoms[ i ].velocity_1.copy( this.atoms[ i ].velocity );
		this.atoms[ i ].omega_1.copy( this.atoms[ i ].omega );

	}

	//時間発展の計算
	this._timeEvolution( dt );
	this.step++;

	this.recordResults();
}

//RK4法アルゴリズムによる時間発展
Molecule.prototype._timeEvolution =  function ( dt ) {

	var time = dt * this.step;

	//力の取得
	var P = this._P.bind( this );
	var V = this._V.bind( this );
	var O = this._O.bind( this );

	//ルンゲ・クッタ法
	var atoms1 = [];
	for( var i=0; i<this.atoms.length; i++ ){
		atoms1[ i ] = {};
		atoms1[ i ].position = this.atoms[ i ].position.clone();
		atoms1[ i ].velocity = this.atoms[ i ].velocity.clone();
		atoms1[ i ].omega = this.atoms[ i ].omega.clone();
	}
	//第一ステップの力の計算
	this.calculateForce( atoms1 , 0 );
	for( var i=0; i < this.atoms.length; i++ ){
		this.atoms[ i ]._k1_r = P( i, atoms1 )
		this.atoms[ i ]._k1_v = V( i, atoms1 )
		this.atoms[ i ]._k1_o = O( i, atoms1 )
	}

	var atoms2 = [];
	for( var i=0; i<this.atoms.length; i++ ){
		atoms2[ i ] = {};
		atoms2[ i ].position = this.atoms[ i ].position.clone().add( this.atoms[ i ]._k1_r.clone().multiplyScalar(dt/2));
		atoms2[ i ].velocity = this.atoms[ i ].velocity.clone().add( this.atoms[ i ]._k1_v.clone().multiplyScalar(dt/2));
		atoms2[ i ].omega    = this.atoms[ i ].omega.clone(   ).add( this.atoms[ i ]._k1_o.clone().multiplyScalar(dt/2));
	}
	//第二ステップの力の計算
	this.calculateForce( atoms2, dt/2 );
	for( var i=0; i < this.atoms.length; i++ ){
		this.atoms[ i ]._k2_r = P( i, atoms2 )
		this.atoms[ i ]._k2_v = V( i, atoms2 )
		this.atoms[ i ]._k2_o = O( i, atoms2 )
	}

	var atoms3 = [];
	for( var i=0; i<this.atoms.length; i++ ){
		atoms3[ i ] = {};
		atoms3[ i ].position = this.atoms[ i ].position.clone().add( this.atoms[ i ]._k2_r.clone().multiplyScalar(dt/2));
		atoms3[ i ].velocity = this.atoms[ i ].velocity.clone().add( this.atoms[ i ]._k2_v.clone().multiplyScalar(dt/2));
		atoms3[ i ].omega    = this.atoms[ i ].omega.clone(   ).add( this.atoms[ i ]._k2_o.clone().multiplyScalar(dt/2));
	}
	//第三ステップの力の計算
	this.calculateForce( atoms3, dt/2 );
	for( var i=0; i < this.atoms.length; i++ ){
		this.atoms[ i ]._k3_r = P( i, atoms3 )
		this.atoms[ i ]._k3_v = V( i, atoms3 )
		this.atoms[ i ]._k3_o = O( i, atoms3 )
	}

	var atoms4 = [];
	for( var i=0; i<this.atoms.length; i++ ){
		atoms4[ i ] = {};
		atoms4[ i ].position = this.atoms[ i ].position.clone().add( this.atoms[ i ]._k3_r.clone().multiplyScalar(dt));
		atoms4[ i ].velocity = this.atoms[ i ].velocity.clone().add( this.atoms[ i ]._k3_v.clone().multiplyScalar(dt));
		atoms4[ i ].omega    = this.atoms[ i ].omega.clone(   ).add( this.atoms[ i ]._k3_o.clone().multiplyScalar(dt));
	}

	//第四ステップの力の計算
	this.calculateForce( atoms4, dt );
	for( var i=0; i < this.atoms.length; i++ ){
		this.atoms[ i ]._k4_r = P( i, atoms4 )
		this.atoms[ i ]._k4_v = V( i, atoms4 )
		this.atoms[ i ]._k4_o = O( i, atoms4 )
	}

	for( var i=0; i<this.atoms.length; i++ ){
		var dr = this.atoms[ i ]._k1_r.add( this.atoms[ i ]._k2_r.multiplyScalar(2) ).add( this.atoms[ i ]._k3_r.multiplyScalar(2) ).add( this.atoms[ i ]._k4_r ).multiplyScalar( dt/6 );
		var dv = this.atoms[ i ]._k1_v.add( this.atoms[ i ]._k2_v.multiplyScalar(2) ).add( this.atoms[ i ]._k3_v.multiplyScalar(2) ).add( this.atoms[ i ]._k4_v ).multiplyScalar( dt/6 );
		var do_= this.atoms[ i ]._k1_o.add( this.atoms[ i ]._k2_o.multiplyScalar(2) ).add( this.atoms[ i ]._k3_o.multiplyScalar(2) ).add( this.atoms[ i ]._k4_o ).multiplyScalar( dt/6 );

		this.atoms[ i ].position.add( dr );
		this.atoms[ i ].velocity.add( dv );
		this.atoms[ i ].omega.add( do_ );
	}

}

//原子-電子間力
Molecule.prototype.calculateForce = function ( atoms, dt ) {

	var time = dt * this.step;

	//重力場の
	for( var i = 0; i < this.atoms.length; i++ ) {
		this.atoms[i].force.set( 0, 0, 0 ).copy( PHYSLAB.g );
		this.atoms[i].torque.set( 0, 0, 0 );

		if(	this.rotationalCoordinateSystem.enable ){

			this.rotationalCoordinateSystem.omega = this.rotationalCoordinateSystem.omega_1 + this.rotationalCoordinateSystem.alpha * dt;

			var omega = new THREE.Vector3( 0, 0, this.rotationalCoordinateSystem.omega);
			var alpha = new THREE.Vector3( 0, 0, this.rotationalCoordinateSystem.alpha);

			//コリオリ力
			var coriolisForce = atoms[ i ].velocity.clone().cross( omega ).multiplyScalar( 2 * this.atoms[i].mass );
			this.atoms[i].force.add( coriolisForce );

			//遠心力
			var centrifugalForce = atoms[ i ].position.clone().cross( omega ).cross( omega ).multiplyScalar( -this.atoms[i].mass );
			this.atoms[i].force.add( centrifugalForce );

			//回転角加速度
			var centrifugalAlphaForce = alpha.clone().cross( atoms[ i ].position ).multiplyScalar( -this.atoms[i].mass );
			this.atoms[i].force.add( centrifugalAlphaForce );



		}

	}



	for( var i = 0; i < this.connects.length; i++ ){

		var n1 = this.connects[ i ].pair1;
		var n2 = this.connects[ i ].pair2;
		var R12 = new THREE.Vector3().subVectors( atoms[ n1 ].position, atoms[ n2 ].position );
		var R12_abs = R12.length();
		var R12_absSq = R12_abs*R12_abs;
		var n12 = R12.clone().normalize();
		var n21 = n12.clone().negate();

		if( this.connects[ i ].type == PHYSLAB.LinearSpringConnection) {

			//ばね弾性力
			var f12 = -this.connects[ i ].k * ( R12_abs - this.connects[ i ].L0 );

		} else if(  this.connects[ i ].type == PHYSLAB.CoulombInteraction ){

			//クーロン力
			var f12 = this.atoms[ n1 ].e * this.atoms[ n2 ].e / ( 4 * Math.PI * this.connects[ i ].epsilon ) / R12_absSq;

		} else if(  this.connects[ i ].type == PHYSLAB.UniversalGravitation ){

			//万有引力
			var f12 = -this.connects[ i ].G * this.atoms[ n1 ].mass * this.atoms[ n2 ].mass / R12_absSq;

		} else if( this.connects[ i ].type == PHYSLAB.LennardJonesPotential ){

			var a = ( this.connects[ i ].sigma / R12_abs );

			//レナード・ジョーンズ相互作用
			var f12 = 24 * this.connects[ i ].epsilon / this.connects[ i ].sigma * ( 2 * Math.pow( a, 13 ) - Math.pow( a, 7 ) ) ;

		}

		this.atoms[ n1 ].force.add ( n12.clone().multiplyScalar( f12 ));
		this.atoms[ n2 ].force.add ( n21.clone().multiplyScalar( f12 ));

	}


	for( var i = 0; i < this.atoms.length; i++ ){
		for( var j = 0; j < this.atoms.length; j++ ){

			this.atoms[ i ].force.add(  this.atoms[ i ].collisionSphereForces[ j ] );
			this.atoms[ i ].torque.add( this.atoms[ i ].collisionSphereTorques[ j ] );

		}
	}

	//剛体棒結合ペアに加わる結合力の計算
	var pairs = this.solidConnectionPairs;
	for(var i = 0; i < pairs.length; i++){

		var length = pairs[ i ].length;

		if( length >= 2 ){

			var ns = [];
			var lines = [];
			for( var i2 = 0; i2 < length; i2++ ){
				//２重配列の準備
				ns[ i2 ] = [];
				for( var j2 = 0; j2 < length; j2++ ){

					var m1 = pairs[ i ][ i2 ];
					var m2 = pairs[ i ][ j2 ];

					var Lij = 0;
					for( var k = 0; k < this.connects.length; k++ ){
						if( (this.connects[ k ].pair1 == m1 && this.connects[ k ].pair2 == m2) || (this.connects[ k ].pair1 == m2 && this.connects[ k ].pair2 == m1)) Lij = this.connects[ k ].L;
					}

					if( Lij ){

						ns[ i2 ][ j2 ] = new THREE.Vector3().subVectors(
							atoms[ m1 ].position,
							atoms[ m2 ].position
						).normalize();

					} else {

						ns[ i2 ][ j2 ] = new THREE.Vector3();

					}

					if( i2 < j2 ) lines.push( [ i2, j2 ] );
				}
			}

			//連立方程式を保持する配列
			var M = [];
			for( var k = 0; k < lines.length; k++ )	M[k] = [];

			for( var line = 0; line < lines.length; line++ ){
				//剛体球番号の取得
				var mi = pairs[ i ][ lines[ line ][ 0 ] ];
				var mj = pairs[ i ][ lines[ line ][ 1 ] ];

				var rij = new THREE.Vector3().subVectors(
					atoms[ mi ].position,
					atoms[ mj ].position
				);
				var vij = new THREE.Vector3().subVectors(
					atoms[ mi ].velocity,
					atoms[ mj ].velocity
				);
				var Fi = this.atoms[ mi ].force.clone().divideScalar( this.atoms[ mi ].mass );
				var Fj = this.atoms[ mj ].force.clone().divideScalar( this.atoms[ mj ].mass );

				if( !this.atoms[ mi ].dynamic ) Fi.set( 0, 0, 0 );
				if( !this.atoms[ mj ].dynamic ) Fj.set( 0, 0, 0 );

				for( var _line = 0; _line < lines.length; _line++ ){

					if( line == _line ){

						M[ line ][ _line ] = rij.length() * ( 1.0 / this.atoms[ mi ].mass + 1.0 / this.atoms[ mj ].mass );

					} else {
						var mi_ =  pairs[ i ][ lines[ _line ][ 0 ] ];
						var mj_ =  pairs[ i ][ lines[ _line ][ 1 ] ];

						M[ line ][ _line ] = rij.length() * ns[ lines[ line ][ 0 ] ][ lines[ line ][ 1 ] ].dot(
							ns[ lines[ _line ][ 0 ] ][ lines[ _line ][ 1 ] ]
						);

						if( lines[ _line ][ 0 ] == lines[ line ][ 0 ] ) M[ line ][ _line ] /=  this.atoms[ mi_ ].mass;
						if( lines[ _line ][ 1 ] == lines[ line ][ 1 ] ) M[ line ][ _line ] /=  this.atoms[ mj_ ].mass;
						if( lines[ _line ][ 0 ] == lines[ line ][ 1 ] ) M[ line ][ _line ] /= -this.atoms[ mi_ ].mass;
						if( lines[ _line ][ 1 ] == lines[ line ][ 0 ] ) M[ line ][ _line ] /= -this.atoms[ mj_ ].mass;

						if( (lines[ _line ][ 0 ] != lines[ line ][ 0 ]) && (lines[ _line ][ 0 ] != lines[ line ][ 1 ]) && (lines[ _line ][ 1 ] != lines[ line ][ 0 ]) && (lines[ _line ][ 1 ] != lines[ line ][ 1 ]) ) M[ line ][ _line ] = 0;

					}

				}

				M[ line ][ lines.length ] = vij.lengthSq() + new THREE.Vector3().subVectors( Fi, Fj ).dot( rij );

			}

			//連立方程式を解く
			var A = solveSimultaneousEquations( M );

			for( var line = 0; line < lines.length; line++ ){
				//剛体球番号の取得
				var mi = pairs[ i ][ lines[ line ][ 0 ] ];
				var mj = pairs[ i ][ lines[ line ][ 1 ] ];

				this.atoms[ mi ].force.add( ns[ lines[ line ][ 0 ] ][ lines[ line ][ 1 ] ].clone().multiplyScalar( -A[ line ] ) );
				this.atoms[ mj ].force.add( ns[ lines[ line ][ 1 ] ][ lines[ line ][ 0 ] ].clone().multiplyScalar( -A[ line ] ) );

			}
		}
	}

	//結合距離の補正
	for( var i = 0; i < this.connects.length; i++ ){
		//結合タイプが剛体棒結合ではない場合はスキップ
		if( this.connects[ i ].type != PHYSLAB.SolidConnection ) continue;

		var ni = this.connects[ i ].pair1;
		var nj = this.connects[ i ].pair2;
		var rij = new THREE.Vector3().subVectors(
			atoms[ ni ].position,
			atoms[ nj ].position
		);
		var vij = new THREE.Vector3().subVectors(
			atoms[ ni ].velocity,
			atoms[ nj ].velocity
		);


		//ズレに比例した弾性力を加算
		var fk = 0.2 * ( rij.length() - this.connects[ i ].L );
		//剛体棒方向速度の減衰
		var fb = 0.05 * vij.dot( rij )

		this.atoms[ ni ].force.add( rij.clone().normalize().multiplyScalar( -fk - fb ));
		this.atoms[ nj ].force.add( rij.clone().normalize().multiplyScalar(  fk + fb ));


	}

}
//
Molecule.prototype._V = function ( n, atoms ) {
//n:原子番号
	if( this.atoms[ n ].dynamic ){

		return this.atoms[ n ].force.clone().divideScalar( this.atoms[ n ].mass );

	} else {

		return new THREE.Vector3();
	}

}
Molecule.prototype._P = function ( n, atoms ) {

	if( this.atoms[ n ].dynamic ){

		return atoms[ n ].velocity;

	} else {

		return new THREE.Vector3();
	}

}
Molecule.prototype._O = function ( n, atoms ) {

	if( this.atoms[ n ].dynamic ){

		return this.atoms[ n ].torque.clone().divideScalar( this.atoms[ n ].moment );

	} else {

		return new THREE.Vector3();
	}

}

//力学的エネルギーの計算
Molecule.prototype.calculateEnergy = function () {
	var kinetic = 0;
	var rotation = 0;
	var potential = 0;
	var rotationalCoordinateSystem = 0;

	for( var i = 0; i < this.atoms.length; i++ ){

		//速度の大きさの２乗の計算
		var v2 = this.atoms[ i ].velocity.lengthSq();
		var o2 = this.atoms[ i ].omega.lengthSq();

		//運動エネルギーの計算
		if( this.atoms[ i ].dynamic ){
			this.atoms[ i ].energy.kinetic =  1.0 / 2.0 * this.atoms[ i ].mass * v2;
			kinetic += this.atoms[ i ].energy.kinetic;

			this.atoms[ i ].energy.potential = -this.atoms[ i ].mass * this.atoms[ i ].position.dot( PHYSLAB.g )  ;
			potential += this.atoms[ i ].energy.potential;

			if( this.rotationalCoordinateSystem.enable ){

				var omega = new THREE.Vector3( 0, 0, this.rotationalCoordinateSystem.omega);

				this.atoms[ i ].energy.rotationalCoordinateSystem = 1 / 2 * this.atoms[ i ].mass * this.atoms[ i ].position.clone().cross( omega ).lengthSq();

				kinetic -= this.atoms[ i ].energy.rotationalCoordinateSystem;

			}

		}
		//回転運動エネルギーの計算
		if( this.atoms[ i ].dynamic && this.atoms[i].moment != Infinity ){

			this.atoms[ i ].energy.rotation = 1.0 / 2.0 * this.atoms[ i ].moment * o2;
			rotation += this.atoms[ i ].energy.rotation;
		}

	}

	for( var i = 0; i < this.connects.length; i++ ){
		this.connects[ i ].potential = 0;

		var n1 = this.connects[ i ].pair1;
		var n2 = this.connects[ i ].pair2;
		var R12 = new THREE.Vector3().subVectors( this.atoms[ n1 ].position, this.atoms[ n2 ].position );
		var R12_abs = R12.length();


		if( this.connects[ i ].type == PHYSLAB.LinearSpringConnection ){

			//ばね弾性エネルギー
			this.connects[ i ].potential = 1/2 *this.connects[ i ].k * ( R12_abs - this.connects[ i ].L0 ) * ( R12_abs - this.connects[ i ].L0 ) ;

		} else if( this.connects[ i ].type == PHYSLAB.CoulombInteraction ){

 			//クーロン相互作用
			this.connects[ i ].potential = this.atoms[ n1 ].e * this.atoms[ n2 ].e / ( 4 * Math.PI * this.connects[ i ].epsilon ) / R12_abs;

		} else if( this.connects[ i ].type == PHYSLAB.UniversalGravitation ){

			//ポテンシャルエネルギーの計算
			this.connects[ i ].potential = - this.connects[ i ].G * this.atoms[ n1 ].mass * this.atoms[ n2 ].mass / R12_abs;

		} else if( this.connects[ i ].type == PHYSLAB.LennardJonesPotential ){

			var a = ( this.connects[ i ].sigma / R12_abs );

			//ポテンシャルエネルギーの計算
			this.connects[ i ].potential = 4 * this.connects[ i ].epsilon * ( Math.pow( a, 12 ) - Math.pow( a, 6 ) );

		}

		potential += this.connects[ i ].potential;
	}

	//力学的エネルギーをオブジェクトで返す
	return { kinetic: kinetic, rotation: rotation, potential: potential };
}

Molecule.prototype.checkCollision = function( dt ){

	for( var i = 0; i < this.atoms.length; i++ ) {
		for( var j = 0; j < this.atoms.length; j++ ) {
			this.atoms[ i ].collisionSphereForces[ j ].set(0,0,0);
			this.atoms[ i ].collisionSphereTorques[ j ].set(0,0,0);
		}
	}

	var collisionFlag = false;

	for( var i = 0; i < this.connects.length; i++ ){

		//結合タイプが剛体球同士の衝突ではない場合はスキップ
		if( this.connects[ i ].type != PHYSLAB.SphereCollision ) continue;

		var ni = this.connects[ i ].pair1;
		var nj = this.connects[ i ].pair2;

		var rij = new THREE.Vector3().subVectors(
			this.atoms[ ni ].position,
			this.atoms[ nj ].position
		);

		//２点間の距離が両者の半径の和をしたまわた場合 → 衝突判定
		if( rij.length() < this.atoms[ ni ].radius + this.atoms[ nj ].radius ){

			//２回連続の衝突は接触と判定：はじめての衝突を判定
			if( !this.atoms[ ni ].collisionSpheres[ nj ] && !this.atoms[ nj ].collisionSpheres[ ni ]){

				//法線ベクトルと接線ベクトルの計算
				var results = this.getCollisionVectors( ni, nj );

				var nij = results.n12;
				var vij = results.v12;

				//衝突力の大きさ
				var F = ( 1 + this.connects[ i ].Er ) * vij.dot( nij ) / ( 1.0 / this.atoms[ ni ].mass + 1.0 / this.atoms[ nj ].mass ) / dt;

				this.atoms[ ni ].collisionSphereForces[ nj ].add( nij.clone().multiplyScalar( -F ) );
				this.atoms[ nj ].collisionSphereForces[ ni ].add( nij.clone().multiplyScalar( F ) );

				/////////////////////////////////////////
				//並進運動量と角運動量の交換を

				//回転判別ベクトル
				var D12 = results.D12;
				//接平面ベクトル（回転力の加わる方向ベクトル）
				var t12 = results.n12;

				var munbo = 1 /this.atoms[ ni ].mass + 1/this.atoms[ nj ].mass + this.atoms[ ni ].radius * this.atoms[ ni ].radius/this.atoms[ ni ].moment + this.atoms[ nj ].radius * this.atoms[ nj ].radius/this.atoms[ nj ].moment;
				var fd12 = - ( 1 + this.connects[ i ].Et ) / munbo / PHYSLAB.dt;

				var fd = D12.clone().multiplyScalar( fd12 );

				var N2 = fd.clone().cross( nij ).multiplyScalar( this.atoms[ nj ].radius );
				var N1 = fd.clone().cross( nij ).multiplyScalar( this.atoms[ ni ].radius );

				this.atoms[ ni ].collisionSphereTorques[ nj ].add( N1 );
				this.atoms[ nj ].collisionSphereTorques[ ni ].add( N2 );

				this.atoms[ ni ].collisionSphereForces[ nj ].add( fd.clone().multiplyScalar( 1 ) );
				this.atoms[ nj ].collisionSphereForces[ ni ].add( fd.clone().multiplyScalar( -1 ) );

				this.atoms[ ni ].collisionSpheres[ nj ] = true;
				this.atoms[ nj ].collisionSpheres[ ni ] = true;

				collisionFlag = true;

			} else {

				this.atoms[ ni ].collisionSpheres[ nj ] = false;
				this.atoms[ nj ].collisionSpheres[ ni ] = false;

			}

		} else {

			this.atoms[ ni ].collisionSpheres[ nj ] = false;
			this.atoms[ nj ].collisionSpheres[ ni ] = false;

		}

	}


	//同時衝突ペアの検証
	var pairs = [];
	pairs[0] = [];

	for( var i = 0; i < this.connects.length; i++ ){

		var ni = this.connects[ i ].pair1;
		var nj = this.connects[ i ].pair2;

		if( this.atoms[ ni ].collisionSpheres[ nj ] ) {

			var c = [];
			for( var j = 0; j < this.solidConnectionPairs.length ; j++ ){

				if( this.solidConnectionPairs[ j ].indexOf( ni ) > -1 ){

					for( var k = 0; k < this.solidConnectionPairs[ j ].length; k++ ){
						if( ni != this.solidConnectionPairs[ j ][ k ]  ) c.push( this.solidConnectionPairs[ j ][ k ] );
					}

				}

				if( this.solidConnectionPairs[ j ].indexOf( nj ) > -1 ){

					for( var k = 0; k < this.solidConnectionPairs[ j ].length; k++ ){
						if( nj != this.solidConnectionPairs[ j ][ k ]  ) c.push( this.solidConnectionPairs[ j ][ k ] );
					}

				}

			}

			for( var k = 0; k < pairs.length; k++ ){

				if( pairs[ k ].indexOf( ni ) >= 0 ){

					for(var m = 0; m < c.length; m++ ){

						if( pairs[ k ].indexOf( c[m] ) == -1 ) pairs[ k ].push( c[m] );

					}

					if( pairs[ k ].indexOf( nj ) == -1 ) pairs[ k ].push( nj );
					else break;

				} else if( pairs[ k ].indexOf( nj ) >= 0 ){

					for(var m = 0; m < c.length; m++ ){

						if( pairs[ k ].indexOf( c[m] ) == -1 ) pairs[ k ].push( c[m] );

					}

					if( pairs[ k ].indexOf( ni ) == -1 ) pairs[ k ].push( ni );
					else break;

				} else if( k == pairs.length - 1 ){

					pairs.push( [ ni, nj ] );

					for (var m = 0; m < c.length; m++ ){

						 if( pairs[ pairs.length - 1 ].indexOf( c[m] ) == -1 ) pairs[ pairs.length - 1 ].push( c[m] );

					}

				}

			}

		}

	}



	for(var i = 0; i < pairs.length; i++){

		if( pairs[ i ].length >= 3 ){

			var length = pairs[ i ].length;
			var ns = [];
			var ts = [];
			var ls = [];
			var lines = [];
			for( var i2 = 0; i2 < length; i2++ ){
				//２重配列の準備
				ns[ i2 ] = [];
				ts[ i2 ] = [];
				ls[ i2 ] = [];
				for( var j2 = 0; j2 < length; j2++ ){

					var m1 = pairs[ i ][ i2 ];
					var m2 = pairs[ i ][ j2 ];

					var index = this.getConnectionIndex( m1, m2 );

					if( index != null ){

						if( this.connects[ index ].type == PHYSLAB.SolidConnection ){

							ns[ i2 ][ j2 ] = new THREE.Vector3();
							ts[ i2 ][ j2 ] = new THREE.Vector3();
							ls[ i2 ][ j2 ] = new THREE.Vector3().subVectors( this.atoms[ m1 ].position, this.atoms[ m2 ].position ).normalize();

						} else if( this.atoms[ m1 ].collisionSpheres[ m2 ] ){

							//法線ベクトルと接線ベクトルの計算
							var results = this.getCollisionVectors( m1, m2 );

							//法線ベクトル（衝突力が加わる方向ベクトル）
							ns[ i2 ][ j2 ] = results.n12;

							//接点の相対速度が小さい場合は接線ベクトルを与えない
							if( results.D12.length() > PHYSLAB.delta) {

								//接平面ベクトル（回転力が加わる方向ベクトル）
								ts[ i2 ][ j2 ] = results.t12;

							} else {

								//接平面ベクトル（回転力が加わる方向ベクトル）
								ts[ i2 ][ j2 ] = new THREE.Vector3();

							}

							ls[ i2 ][ j2 ] = new THREE.Vector3();

						} else {

							ns[ i2 ][ j2 ] = new THREE.Vector3();
							ts[ i2 ][ j2 ] = new THREE.Vector3();
							ls[ i2 ][ j2 ] = new THREE.Vector3();

						}

						if( i2 < j2 ) lines.push( [ i2, j2 ] );

					} else {

						ns[ i2 ][ j2 ] = new THREE.Vector3();
						ts[ i2 ][ j2 ] = new THREE.Vector3();
						ls[ i2 ][ j2 ] = new THREE.Vector3();

					}
				}
			}

			//連立方程式を保持する配列
			var M = []
			var NL = lines.length;
			for( var  k= 0; k < 3 * lines.length; k++ )	{

				M[ k ] = [];

			}

			for( var line = 0; line < lines.length; line++ ){
				//剛体球番号の取得
				var mi = pairs[ i ][ lines[ line ][ 0 ] ];
				var mj = pairs[ i ][ lines[ line ][ 1 ] ];

				var rij = new THREE.Vector3().subVectors( this.atoms[ mi ].position, this.atoms[ mj ].position );

				for( var _line = 0; _line < lines.length; _line++ ){

					if( line == _line ){

						//衝突力条件
						M[ line ][ _line ] = dt * ( 1.0 / this.atoms[ mi ].mass + 1.0 / this.atoms[ mj ].mass );

						M[ line ][ NL + _line ] = dt * ns[ lines[ line ][ 0 ] ][ lines[ line ][ 1 ] ].dot(
							ts[ lines[ _line ][ 0 ] ][ lines[ _line ][ 1 ] ] 
						) * ( 1.0 / this.atoms[ mi ].mass + 1.0 / this.atoms[ mj ].mass );

						M[ line ][ 2 * NL + _line ] = dt * ns[ lines[ line ][ 0 ] ][ lines[ line ][ 1 ] ].dot(  
							ls[ lines[ _line ][ 0 ] ][ lines[ _line ][ 1 ] ] 
						) * ( 1.0 / this.atoms[ mi ].mass + 1.0 / this.atoms[ mj ].mass );

						//回転力条件
						M[ NL + line ][ _line ] = dt * ts[ lines[ line ][ 0 ] ][ lines[ line ][ 1 ] ].dot(  
							ns[ lines[ _line ][ 0 ] ][ lines[ _line ][ 1 ] ] 
						) * ( 1.0 / this.atoms[ mi ].mass + 1.0 / this.atoms[ mj ].mass );

						M[ NL + line ][ NL + _line ] = dt * ( 1.0 / this.atoms[ mi ].mass + 1.0 / this.atoms[ mj ].mass 
							+ this.atoms[ mi ].radius * this.atoms[ mi ].radius / this.atoms[ mi ].moment 
							+ this.atoms[ mj ].radius * this.atoms[ mj ].radius / this.atoms[ mj ].moment
						);

						M[ NL + line ][ 2 * NL + _line ] = dt * ts[ lines[ line ][ 0 ] ][ lines[ line ][ 1 ] ].dot(  
							ls[ lines[ _line ][ 0 ] ][ lines[ _line ][ 1 ] ] 
						) * ( 1.0 / this.atoms[ mi ].mass + 1.0 / this.atoms[ mj ].mass );

						//拘束力条件
						M[ 2 * NL + line ][ _line ] = dt * ls[ lines[ line ][ 0 ] ][ lines[ line ][ 1 ] ].dot(  
							ns[ lines[ _line ][ 0 ] ][ lines[ _line ][ 1 ] ] 
						) * ( 1.0 / this.atoms[ mi ].mass + 1.0 / this.atoms[ mj ].mass );

						M[ 2 * NL + line ][ NL + _line ] = dt * ls[ lines[ line ][ 0 ] ][ lines[ line ][ 1 ] ].dot(  
							ts[ lines[ _line ][ 0 ] ][ lines[ _line ][ 1 ] ] 
						) * ( 1.0 / this.atoms[ mi ].mass + 1.0 / this.atoms[ mj ].mass );

						M[ 2 * NL + line ][ 2* NL + _line ] = dt * ( 1.0 / this.atoms[ mi ].mass + 1.0 / this.atoms[ mj ].mass );

					} else {

						var mi_ =  pairs[ i ][ lines[ _line ][ 0 ] ];
						var mj_ =  pairs[ i ][ lines[ _line ][ 1 ] ];

						M[ line ][ _line ] = dt * ns[ lines[ line ][ 0 ] ][ lines[ line ][ 1 ] ].dot(  
							ns[ lines[ _line ][ 0 ] ][ lines[ _line ][ 1 ] ] 
						);

						M[ line ][ NL + _line ] = dt * ns[ lines[ line ][ 0 ] ][ lines[ line ][ 1 ] ].dot(  
							ts[ lines[ _line ][ 0 ] ][ lines[ _line ][ 1 ] ] 
						);

						M[ line ][ 2 * NL + _line ] = dt * ns[ lines[ line ][ 0 ] ][ lines[ line ][ 1 ] ].dot(  
							ls[ lines[ _line ][ 0 ] ][ lines[ _line ][ 1 ] ] 
						); 


						M[ NL + line ][ _line ] = dt * ts[ lines[ line ][ 0 ] ][ lines[ line ][ 1 ] ].dot(  
							ns[ lines[ _line ][ 0 ] ][ lines[ _line ][ 1 ] ] 
						); 

						M[ NL + line ][ NL + _line ] = dt;

						M[ NL + line ][ 2 * NL + _line ] = dt * ts[ lines[ line ][ 0 ] ][ lines[ line ][ 1 ] ].dot(  
							ls[ lines[ _line ][ 0 ] ][ lines[ _line ][ 1 ] ] 
						); 


						M[ 2 * NL + line ][ _line ] = dt * ls[ lines[ line ][ 0 ] ][ lines[ line ][ 1 ] ].dot(  
							ns[ lines[ _line ][ 0 ] ][ lines[ _line ][ 1 ] ] 
						);

						M[ 2 * NL + line ][ NL + _line ] = dt * ls[ lines[ line ][ 0 ] ][ lines[ line ][ 1 ] ].dot(  
							ts[ lines[ _line ][ 0 ] ][ lines[ _line ][ 1 ] ] 
						); 

						M[ 2 * NL + line ][ 2 * NL + _line ] = dt * ls[ lines[ line ][ 0 ] ][ lines[ line ][ 1 ] ].dot(  
							ls[ lines[ _line ][ 0 ] ][ lines[ _line ][ 1 ] ] 
						);

						if( lines[ _line ][ 0 ] == lines[ line ][ 0 ] ) {

							M[ line ][ _line ] /= this.atoms[ mi_ ].mass;
							M[ line ][ NL + _line ] /= this.atoms[ mi_ ].mass;
							M[ line ][ 2 * NL + _line ] /= this.atoms[ mi_ ].mass;

							M[ NL + line ][ _line ] /= this.atoms[ mi_ ].mass;
							M[ NL + line ][ NL + _line ] *= ts[ lines[ line ][ 0 ] ][ lines[ line ][ 1 ] ].dot(  ts[ lines[ _line ][ 0 ] ][ lines[ _line ][ 1 ] ] ) / this.atoms[ mi_ ].mass + ns[ lines[ _line ][ 0 ] ][ lines[ _line ][ 1 ] ].clone().cross( ts[ lines[ _line ][ 0 ] ][ lines[ _line ][ 1 ] ] ).cross( ns[ lines[ line ][ 0 ] ][ lines[ line ][ 1 ] ] ).dot( ts[ lines[ line ][ 0 ] ][ lines[ line ][ 1 ] ] ) * this.atoms[ mi_ ].radius * this.atoms[ mi_ ].radius / this.atoms[ mi_ ].moment;
							M[ NL + line ][ 2 * NL + _line ] /= this.atoms[ mi_ ].mass;


							M[ 2 * NL + line ][ _line ] /= this.atoms[ mi_ ].mass;
							M[ 2 * NL + line ][ NL + _line ] /= this.atoms[ mi_ ].mass;
							M[ 2 * NL + line ][ 2 * NL + _line ] /= this.atoms[ mi_ ].mass;

						}

						if( lines[ _line ][ 1 ] == lines[ line ][ 1 ] ){

							M[ line ][ _line ] /= this.atoms[ mj_ ].mass;
							M[ line ][ NL + _line ] /= this.atoms[ mj_ ].mass;
							M[ line ][ 2 * NL + _line ] /= this.atoms[ mj_ ].mass;

							M[ NL + line ][ _line ] /= this.atoms[ mj_ ].mass;

							M[ NL + line ][ NL + _line ] *= ts[ lines[ line ][ 0 ] ][ lines[ line ][ 1 ] ].dot(  ts[ lines[ _line ][ 0 ] ][ lines[ _line ][ 1 ] ] ) / this.atoms[ mj_ ].mass + ns[ lines[ _line ][ 0 ] ][ lines[ _line ][ 1 ] ].clone().cross( ts[ lines[ _line ][ 0 ] ][ lines[ _line ][ 1 ] ] ).cross( ns[ lines[ line ][ 0 ] ][ lines[ line ][ 1 ] ] ).dot( ts[ lines[ line ][ 0 ] ][ lines[ line ][ 1 ] ] ) * this.atoms[ mj_ ].radius * this.atoms[ mj_ ].radius / this.atoms[ mj_ ].moment;
							M[ NL + line ][ 2 * NL + _line ] /= this.atoms[ mj_ ].mass;

							M[ 2 * NL + line ][ _line ] /= this.atoms[ mj_ ].mass;
							M[ 2 * NL + line ][ NL + _line ] /= this.atoms[ mj_ ].mass;
							M[ 2 * NL + line ][ 2 * NL + _line ] /= this.atoms[ mj_ ].mass;

						}
						if( lines[ _line ][ 0 ] == lines[ line ][ 1 ] ){

							M[ line ][ _line ] /= -this.atoms[ mi_ ].mass;
							M[ line ][ NL + _line ] /= -this.atoms[ mi_ ].mass;
							M[ line ][ 2 * NL + _line ] /= -this.atoms[ mi_ ].mass;

							M[ NL + line ][ _line ] /= -this.atoms[ mi_ ].mass;
							M[ NL + line ][ NL + _line ] *= +ts[ lines[ line ][ 0 ] ][ lines[ line ][ 1 ] ].dot(  ts[ lines[ _line ][ 0 ] ][ lines[ _line ][ 1 ] ] ) / this.atoms[ mi_ ].mass + ns[ lines[ _line ][ 0 ] ][ lines[ _line ][ 1 ] ].clone().cross( ts[ lines[ _line ][ 0 ] ][ lines[ _line ][ 1 ] ] ).cross( ns[ lines[ line ][ 0 ] ][ lines[ line ][ 1 ] ] ).dot( ts[ lines[ line ][ 0 ] ][ lines[ line ][ 1 ] ] ) * this.atoms[ mi_ ].radius * this.atoms[ mi_ ].radius / this.atoms[ mi_ ].moment;
							M[ NL + line ][ 2 * NL + _line ] /= -this.atoms[ mi_ ].mass;

//M[ NL + line ][ NL + _line ]の符号に注意

							M[ 2 * NL + line ][ _line ] /= -this.atoms[ mi_ ].mass;
							M[ 2 * NL + line ][ NL + _line ] /= -this.atoms[ mi_ ].mass;
							M[ 2 * NL + line ][ 2 * NL + _line ] /= -this.atoms[ mi_ ].mass;

						}
						if( lines[ _line ][ 1 ] == lines[ line ][ 0 ] ){

							M[ line ][ _line ] /= -this.atoms[ mj_ ].mass;
							M[ line ][ NL + _line ] /= -this.atoms[ mj_ ].mass;
							M[ line ][ 2 * NL + _line ] /= -this.atoms[ mj_ ].mass;

							M[ NL + line ][ _line ] /= -this.atoms[ mj_ ].mass;
							M[ NL + line ][ NL + _line ] *= +ts[ lines[ line ][ 0 ] ][ lines[ line ][ 1 ] ].dot(  ts[ lines[ _line ][ 0 ] ][ lines[ _line ][ 1 ] ] ) / this.atoms[ mj_ ].mass + ns[ lines[ _line ][ 0 ] ][ lines[ _line ][ 1 ] ].clone().cross( ts[ lines[ _line ][ 0 ] ][ lines[ _line ][ 1 ] ] ).cross( ns[ lines[ line ][ 0 ] ][ lines[ line ][ 1 ] ] ).dot( ts[ lines[ line ][ 0 ] ][ lines[ line ][ 1 ] ] ) * this.atoms[ mj_ ].radius * this.atoms[ mj_ ].radius / this.atoms[ mj_ ].moment;

//M[ NL + line ][ NL + _line ]の符号に注意

							M[ NL + line ][ 2 * NL + _line ] /= -this.atoms[ mj_ ].mass;

							M[ 2 * NL + line ][ _line ] /= -this.atoms[ mj_ ].mass;
							M[ 2 * NL + line ][ NL + _line ] /= -this.atoms[ mj_ ].mass;
							M[ 2 * NL + line ][ 2 * NL + _line ] /= -this.atoms[ mj_ ].mass;
						}

						if( (lines[ _line ][ 0 ] != lines[ line ][ 0 ]) && (lines[ _line ][ 0 ] != lines[ line ][ 1 ]) && (lines[ _line ][ 1 ] != lines[ line ][ 0 ]) && (lines[ _line ][ 1 ] != lines[ line ][ 1 ]) ) {

							M[ line ][ _line ] = 0
							M[ line ][ NL + _line ] = 0
							M[ line ][ 2 * NL + _line ] = 0
							M[ NL + line ][ _line ] = 0
							M[ NL + line ][ NL + _line ] = 0
							M[ NL + line ][ 2 * NL + _line ] = 0
							M[ 2 * NL + line ][ _line ] = 0
							M[ 2 * NL + line ][ NL + _line ] = 0
							M[ 2 * NL + line ][ 2 * NL + _line ] = 0

						}

					}

				}

				var index = this.getConnectionIndex( mi, mj );

				if( this.connects[ index ].type == PHYSLAB.SphereCollision && this.atoms[ mi ].collisionSpheres[ mj ] ){

					var Er = this.connects[ index ].Er;
					var Et = this.connects[ index ].Et;

					M[ line ][ 3 * NL ] = - ( 1 + Er ) * new THREE.Vector3().subVectors( 
						this.atoms[ mi ].velocity, this.atoms[ mj ].velocity 
					).dot( ns[ lines[ line ][ 0 ] ][ lines[ line ][ 1 ] ] ) ;

					//法線ベクトルと接線ベクトルの計算
					var results = this.getCollisionVectors( mi, mj );

					var D12 = results.D12;

					M[ NL + line ][ 3 * NL ] = - ( 1 + Et ) * D12.dot( ts[ lines[ line ][ 0 ] ][ lines[ line ][ 1 ] ]  );


				} else {

					M[ line ][ 3 * NL ] = 0;
					M[ NL + line ][ 3 * NL ] = 0;

				}

				M[ 2 * NL + line ][ 3 * NL ] = 0;

			}

//			console.log( ns );
//			console.log( ts );
//			console.log( M );


			//連立方程式を解く
			var A = solveSimultaneousEquations( M );

//			console.log( A );

			for( var line = 0; line < lines.length; line++ ){
				//剛体球番号の取得
				var mi = pairs[ i ][ lines[ line ][ 0 ] ];
				var mj = pairs[ i ][ lines[ line ][ 1 ] ];

				//衝突力の設定
				this.atoms[ mi ].collisionSphereForces[ mj ].set(0,0,0).add( ns[ lines[ line ][ 0 ] ][ lines[ line ][ 1 ] ].clone().multiplyScalar( A[ line ] ) );
				this.atoms[ mj ].collisionSphereForces[ mi ].set(0,0,0).add( ns[ lines[ line ][ 1 ] ][ lines[ line ][ 0 ] ].clone().multiplyScalar( A[ line ] ) );

				//接線方向の力の加算
				this.atoms[ mi ].collisionSphereForces[ mj ].add( ts[ lines[ line ][ 0 ] ][ lines[ line ][ 1 ] ].clone().multiplyScalar( A[ NL + line ] ) );
				this.atoms[ mj ].collisionSphereForces[ mi ].add( ts[ lines[ line ][ 1 ] ][ lines[ line ][ 0 ] ].clone().multiplyScalar( A[ NL + line ] ) );

				var N1 = ts[ lines[ line ][ 0 ] ][ lines[ line ][ 1 ] ].clone().cross( ns[ lines[ line ][ 0 ] ][ lines[ line ][ 1 ] ] ).multiplyScalar( this.atoms[ mi ].radius * A[ NL + line ]);
				var N2 = ts[ lines[ line ][ 1 ] ][ lines[ line ][ 0 ] ].clone().cross( ns[ lines[ line ][ 1 ] ][ lines[ line ][ 0 ] ] ).multiplyScalar( this.atoms[ mj ].radius * A[ NL + line ]);

				//トルクの設定
				this.atoms[ mi ].collisionSphereTorques[ mj ].set(0,0,0).add( N1 );
				this.atoms[ mj ].collisionSphereTorques[ mi ].set(0,0,0).add( N2 );

				//結合力の加算
				this.atoms[ mi ].collisionSphereForces[ mj ].add( ls[ lines[ line ][ 0 ] ][ lines[ line ][ 1 ] ].clone().multiplyScalar( A[ 2 * NL + line ] ) );
				this.atoms[ mj ].collisionSphereForces[ mi ].add( ls[ lines[ line ][ 1 ] ][ lines[ line ][ 0 ] ].clone().multiplyScalar( A[ 2 * NL + line ] ) );

			}

		}

	}

	if( collisionFlag ){

		for( var i = 0; i < this.atoms.length; i++ ) {

			//時間を巻き戻す
			this.atoms[ i ].position.copy( this.atoms[ i ].position_1 );
			this.atoms[ i ].velocity.copy( this.atoms[ i ].velocity_1 );
			this.atoms[ i ].omega.copy( this.atoms[ i ].omega_1 );

		}

		this.step--;

	}
/*
	//２回連続の衝突時は時間を1/100にして再計算
	if( collisionFlag && this.collisionMode ){

		console.log(this.step, "collisionMode")

		var N = 100;
		this.collisionMode = false;

		for( var i = 0; i < N; i++ ){

			molecule.checkCollision( PHYSLAB.dt/N );
			molecule.timeEvolution( PHYSLAB.dt/N );
			this.step--;

		}
		this.step++;


	} else if( collisionFlag ){

		this.collisionMode = true;

	} else {

		this.collisionMode = false;

	}
*/
}
//結合情報
Molecule.prototype.getConnectionIndex = function( m1, m2 ){

	for( var k = 0; k < this.connects.length; k++ ){
		if( (this.connects[ k ].pair1 == m1 && this.connects[ k ].pair2 == m2) || 
			(this.connects[ k ].pair1 == m2 && this.connects[ k ].pair2 == m1)) {
			return k;
			break;
		}
	}
	return null;
}

//衝突時に必要なベクトル
Molecule.prototype.getCollisionVectors = function( m1, m2 ){

	var results = {};

	//衝突時の位置と速度の補正
	var r1 = new THREE.Vector3().addVectors( this.atoms[ m1 ].position, this.atoms[ m1 ].position_1 ).multiplyScalar(0.5);
	var r2 = new THREE.Vector3().addVectors( this.atoms[ m2 ].position, this.atoms[ m2 ].position_1 ).multiplyScalar(0.5);
	var r12 = new THREE.Vector3().subVectors( r1, r2 );
	var v1 = new THREE.Vector3().addVectors( this.atoms[ m1 ].velocity, this.atoms[ m1 ].velocity_1 ).multiplyScalar(0.5);
	var v2 = new THREE.Vector3().addVectors( this.atoms[ m2 ].velocity, this.atoms[ m2 ].velocity_1 ).multiplyScalar(0.5);
	var v12 = new THREE.Vector3().subVectors( v1, v2 );

	results.r12 = r12;
	results.v12 = v12;

	var n12 = r12.clone().normalize();
	results.n12 = n12;

	//角加速度ベクトルの差
	var omega12 = new THREE.Vector3().addVectors( this.atoms[ m1 ].omega, this.atoms[ m2 ].omega );
	var omega1_a = this.atoms[ m1 ].omega.clone().multiplyScalar( this.atoms[ m1 ].radius );
	var omega2_a = this.atoms[ m2 ].omega.clone().multiplyScalar( this.atoms[ m2 ].radius );
	var omega_a = omega1_a.clone().add( omega2_a );
	var omega1_v = omega1_a.clone().cross( n12 );
	var omega2_v = omega2_a.clone().cross( n12 );
	var omega_v = omega1_v.clone().add( omega2_v );

	var v12_t = v12.clone().sub( n12.clone().multiplyScalar( v12.dot( n12 ) ) );
	var D12 = v12_t.sub( omega_v );

	results.D12 = D12;

	//接平面ベクトル（回転力の加わる方向ベクトル）
	var t12 = D12.clone().normalize();

	results.t12 = t12;

	return results;
}

/////////////////////////////////////////////////////////////////////
//ガウス法による連立方程式の解法
var solveSimultaneousEquations = function( M ){

	// 連立数の取得
	var n = M.length;

	// 前進消去（ピボット操作あり）
	for( var k = 0; k < n-1; k++ ){

		var p = k;
		var max = Math.abs( M[k][k] );

		for( var i = k + 1; i < n; i++){  // ピボット選択
			if( Math.abs( M[i][k] ) > max){
				p = i;
				max = Math.abs( M[i][k] );
			}
		}
		if( Math.abs( max ) < 1E-12){
			var ans = [];
			for( var k = 0; k < n; k++ ) ans[k] = 0;
			console.log("前進消去時のピボットが小さすぎます（方程式の数が足りない可能性があります）");
			return ans;
		}
		if( p != k ){  
			for(var i = k; i <= n; i++){
				var tmp = M[k][i];
				M[k][i] = M[p][i];
				M[p][i] = tmp;
			}
		}


		for( var i = k + 1; i < n; i++){
			for( var j = k + 1; j <= n; j++){
				M[i][j] = M[i][j] - M[k][j] * M[i][k] / M[k][k];
			}
		}
	}

	//連立方程式の解を格納する配列
	var ans = [];
	// 後退代入
	for( var k = n - 1; k >= 0; k--){
		for( var j = k + 1; j < n; j++){
			M[k][n] = M[k][n] - M[k][j] * ans[j];
		}
		if( Math.abs( M[k][k] ) < 1E-12){
			console.log("前進消去時のピボットが小さすぎます（方程式の数が多すぎる可能性があります）");
			ans[k] = 0;
		} else {
			ans[k] = M[k][n] / M[k][k];
		}		

	}

	return ans;
}